@@ -12,16 +12,16 @@ pub mod application;
 #[cfg(feature = "application")]
 pub use application::{handler, route, scope, Application, Context, Error, FromContext};
 
-#[doc(hidden)]
-#[cfg(feature = "askama")]
-#[cfg_attr(docsrs, doc(cfg(feature = "askama")))]
-pub mod askama;
-
 #[cfg(feature = "cookies")]
 #[cfg_attr(docsrs, doc(cfg(feature = "cookies")))]
 /// Cookie support
 pub mod cookies;
 
+#[cfg(feature = "key")]
+#[cfg_attr(docsrs, doc(cfg(feature = "key")))]
+/// AEAD encryption/decryption support
+pub mod key;
+
 #[cfg(feature = "forms")]
 #[cfg_attr(docsrs, doc(cfg(feature = "forms")))]
 /// Form generation and data validation
