@@ -2,7 +2,7 @@
 
 use async_trait::async_trait;
 use hyper::Body;
-use mendes::application::Responder;
+use mendes::application::IntoResponse;
 use mendes::http::request::Parts;
 use mendes::http::{Response, StatusCode};
 use mendes::{handler, route, Application, Context};
@@ -48,7 +48,7 @@ impl From<&Error> for StatusCode {
     }
 }
 
-impl Responder<App> for Error {
+impl IntoResponse<App> for Error {
     fn into_response(self, _: &App, _: &Parts) -> Response<Body> {
         let Error::Mendes(err) = self;
         Response::builder()
